@@ -14,23 +14,21 @@
 //
 ///////////////////////////////////////////////////////////////////////////////
 
-#include <cstdlib>      // for std::exit
-#include <gsl/gsl_util> // for narrow
+#include <cstdlib>        // for std::exit
+#include <gsl/gsl_assert> // for get_terminate
+#include <gsl/gsl_util>   // for narrow
 
 int narrow_no_throw()
 {
-    long long bigNumber = 0x0fffffffffffffff;
-    return gsl::narrow<int>(bigNumber); 
+    const long long bigNumber = 0x0fffffffffffffff;
+    return gsl::narrow<int>(bigNumber);
 }
 
-void test_terminate()
-{ 
-    std::exit(0);
-}
+[[noreturn]] void test_terminate() { std::exit(0); }
 
-void setup_termination_handler()
+void setup_termination_handler() noexcept
 {
-#if defined(_MSC_VER)
+#if defined(GSL_MSVC_USE_STL_NOEXCEPTION_WORKAROUND)
 
     auto& handler = gsl::details::get_terminate_handler();
     handler = &test_terminate;
@@ -42,7 +40,6 @@ void setup_termination_handler()
 #endif
 }
 
-
 int main()
 {
     setup_termination_handler();
