@@ -14,6 +14,13 @@
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+#ifdef _MSC_VER
+// blanket turn off warnings from CppCoreCheck from catch
+// so people aren't annoyed by them when running the tool.
+#pragma warning(disable : 26440 26426 26497) // from catch
+
+#endif
+
 #include <catch/catch.hpp> // for AssertionHandler, StringRef, CHECK, TEST_...
 
 #include <gsl/gsl_byte> // for byte
@@ -30,9 +37,10 @@
 #include <type_traits> // for integral_constant<>::value, is_default_co...
 #include <vector>      // for vector
 
-namespace gsl {
+namespace gsl
+{
 struct fail_fast;
-}  // namespace gsl
+} // namespace gsl
 
 using namespace std;
 using namespace gsl;
@@ -45,8 +53,16 @@ struct BaseClass
 struct DerivedClass : BaseClass
 {
 };
-}
+struct AddressOverloaded
+{
+#if (__cplusplus > 201402L)
+    [[maybe_unused]]
+#endif
+    AddressOverloaded operator&() const { return {}; }
+};
+} // namespace
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("default_constructor")
 {
     {
@@ -81,6 +97,7 @@ TEST_CASE("default_constructor")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("size_optimization")
 {
     {
@@ -94,56 +111,60 @@ TEST_CASE("size_optimization")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("from_nullptr_size_constructor")
 {
     {
-        span<int> s{nullptr, static_cast<span<int>::index_type>(0)};
+        span<int> s{nullptr, narrow_cast<span<int>::index_type>(0)};
         CHECK((s.size() == 0 && s.data() == nullptr));
 
-        span<const int> cs{nullptr, static_cast<span<int>::index_type>(0)};
+        span<const int> cs{nullptr, narrow_cast<span<int>::index_type>(0)};
         CHECK((cs.size() == 0 && cs.data() == nullptr));
     }
 
     {
-        span<int, 0> s{nullptr, static_cast<span<int>::index_type>(0)};
+        span<int, 0> s{nullptr, narrow_cast<span<int>::index_type>(0)};
         CHECK((s.size() == 0 && s.data() == nullptr));
 
-        span<const int, 0> cs{nullptr, static_cast<span<int>::index_type>(0)};
+        span<const int, 0> cs{nullptr, narrow_cast<span<int>::index_type>(0)};
         CHECK((cs.size() == 0 && cs.data() == nullptr));
     }
 
     {
         auto workaround_macro = []() {
-            span<int, 1> s{nullptr, static_cast<span<int>::index_type>(0)};
+            const span<int, 1> s{nullptr, narrow_cast<span<int>::index_type>(0)};
         };
         CHECK_THROWS_AS(workaround_macro(), fail_fast);
     }
 
     {
-        auto workaround_macro = []() { span<int> s{nullptr, 1}; };
+        auto workaround_macro = []() { const span<int> s{nullptr, 1}; };
         CHECK_THROWS_AS(workaround_macro(), fail_fast);
 
-        auto const_workaround_macro = []() { span<const int> cs{nullptr, 1}; };
+        auto const_workaround_macro = []() { const span<const int> cs{nullptr, 1}; };
         CHECK_THROWS_AS(const_workaround_macro(), fail_fast);
     }
 
     {
-        auto workaround_macro = []() { span<int, 0> s{nullptr, 1}; };
+        auto workaround_macro = []() { const span<int, 0> s{nullptr, 1}; };
         CHECK_THROWS_AS(workaround_macro(), fail_fast);
 
-        auto const_workaround_macro = []() { span<const int, 0> s{nullptr, 1}; };
+        auto const_workaround_macro = []() { const span<const int, 0> s{nullptr, 1}; };
         CHECK_THROWS_AS(const_workaround_macro(), fail_fast);
     }
 
     {
-        span<int*> s{nullptr, static_cast<span<int>::index_type>(0)};
+        span<int*> s{nullptr, narrow_cast<span<int>::index_type>(0)};
         CHECK((s.size() == 0 && s.data() == nullptr));
 
-        span<const int*> cs{nullptr, static_cast<span<int>::index_type>(0)};
+        span<const int*> cs{nullptr, narrow_cast<span<int>::index_type>(0)};
         CHECK((cs.size() == 0 && cs.data() == nullptr));
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.2) // NO-FORMAT: attribute
 TEST_CASE("from_pointer_length_constructor")
 {
     int arr[4] = {1, 2, 3, 4};
@@ -164,7 +185,7 @@ TEST_CASE("from_pointer_length_constructor")
                 }
             }
             {
-                span<int> s = { &arr[i], 4-i };
+                span<int> s = { &arr[i], 4-narrow_cast<ptrdiff_t>(i) };
                 CHECK(s.size() == 4-i);
                 CHECK(s.data() == &arr[i]);
                 CHECK(s.empty() == (4-i == 0));
@@ -186,13 +207,13 @@ TEST_CASE("from_pointer_length_constructor")
 
     {
         int* p = nullptr;
-        span<int> s{p, static_cast<span<int>::index_type>(0)};
+        span<int> s{p, narrow_cast<span<int>::index_type>(0)};
         CHECK((s.size() == 0 && s.data() == nullptr));
     }
 
     {
         int* p = nullptr;
-        auto workaround_macro = [=]() { span<int> s{p, 2}; };
+        auto workaround_macro = [=]() { const span<int> s{p, 2}; };
         CHECK_THROWS_AS(workaround_macro(), fail_fast);
     }
 
@@ -204,7 +225,7 @@ TEST_CASE("from_pointer_length_constructor")
 
     {
         int* p = nullptr;
-        auto s = make_span(p, static_cast<span<int>::index_type>(0));
+        auto s = make_span(p, narrow_cast<span<int>::index_type>(0));
         CHECK((s.size() == 0 && s.data() == nullptr));
     }
 
@@ -215,6 +236,8 @@ TEST_CASE("from_pointer_length_constructor")
     }
 }
 
+
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("from_pointer_pointer_constructor")
 {
     int arr[4] = {1, 2, 3, 4};
@@ -296,12 +319,12 @@ TEST_CASE("from_array_constructor")
     int arr[5] = {1, 2, 3, 4, 5};
 
     {
-        span<int> s{arr};
+        const span<int> s{arr};
         CHECK((s.size() == 5 && s.data() == &arr[0]));
     }
 
     {
-        span<int, 5> s{arr};
+        const span<int, 5> s{arr};
         CHECK((s.size() == 5 && s.data() == &arr[0]));
     }
 
@@ -333,8 +356,8 @@ TEST_CASE("from_array_constructor")
     }
 #endif
     {
-        span<int[3]> s{&(arr2d[0]), 1};
-        CHECK((s.size() == 1 && s.data() == &arr2d[0]));
+        const span<int[3]> s{std::addressof(arr2d[0]), 1};
+        CHECK((s.size() == 1 && s.data() == std::addressof(arr2d[0])));
     }
 
     int arr3d[2][3][2] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};
@@ -362,26 +385,38 @@ TEST_CASE("from_array_constructor")
     }
 #endif
     {
-        span<int[3][2]> s{&arr3d[0], 1};
-        CHECK((s.size() == 1 && s.data() == &arr3d[0]));
+        const span<int[3][2]> s{std::addressof(arr3d[0]), 1};
+        CHECK((s.size() == 1 && s.data() == std::addressof(arr3d[0])));
     }
 
     {
-        auto s = make_span(arr);
-        CHECK((s.size() == 5 && s.data() == &arr[0]));
+        const auto s = make_span(arr);
+        CHECK((s.size() == 5 && s.data() == std::addressof(arr[0])));
     }
 
     {
-        auto s = make_span(&(arr2d[0]), 1);
-        CHECK((s.size() == 1 && s.data() == &arr2d[0]));
+        const auto s = make_span(std::addressof(arr2d[0]), 1);
+        CHECK((s.size() == 1 && s.data() == std::addressof(arr2d[0])));
     }
 
     {
-        auto s = make_span(&arr3d[0], 1);
-        CHECK((s.size() == 1 && s.data() == &arr3d[0]));
+        const auto s = make_span(std::addressof(arr3d[0]), 1);
+        CHECK((s.size() == 1 && s.data() == std::addressof(arr3d[0])));
     }
+
+    AddressOverloaded ao_arr[5] = {};
+
+    {
+        const span<AddressOverloaded, 5> s{ao_arr};
+        CHECK((s.size() == 5 && s.data() == std::addressof(ao_arr[0])));
+    }
+
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(r.11) // NO-FORMAT: attribute
+GSL_SUPPRESS(i.11) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.1) // NO-FORMAT: attribute
 TEST_CASE("from_dynamic_array_constructor")
 {
     double(*arr)[3][4] = new double[100][3][4];
@@ -399,6 +434,8 @@ TEST_CASE("from_dynamic_array_constructor")
     delete[] arr;
 }
 
+
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("from_std_array_constructor")
 {
     std::array<int, 4> arr = {1, 2, 3, 4};
@@ -419,6 +456,19 @@ TEST_CASE("from_std_array_constructor")
         CHECK((cs.size() == narrow_cast<ptrdiff_t>(arr.size()) && cs.data() == arr.data()));
     }
 
+    {
+        std::array<int, 0> empty_arr{};
+        span<int> s{empty_arr};
+        CHECK((s.size() == 0 && s.empty()));
+    }
+
+    std::array<AddressOverloaded, 4> ao_arr{};
+
+    {
+        span<AddressOverloaded, 4> fs{ao_arr};
+        CHECK((fs.size() == narrow_cast<ptrdiff_t>(ao_arr.size()) && ao_arr.data() == fs.data()));
+    }
+
 #ifdef CONFIRM_COMPILATION_ERRORS
     {
         span<int, 2> s{arr};
@@ -459,8 +509,27 @@ TEST_CASE("from_std_array_constructor")
         auto s = make_span(arr);
         CHECK((s.size() == narrow_cast<ptrdiff_t>(arr.size()) && s.data() == arr.data()));
     }
+
+    // This test checks for the bug found in gcc 6.1, 6.2, 6.3, 6.4, 6.5 7.1, 7.2, 7.3 - issue #590
+    {
+        span<int> s1 = make_span(arr);
+
+        static span<int> s2;
+        s2 = s1;
+
+    #if defined(__GNUC__) && __GNUC__ == 6 && (__GNUC_MINOR__ == 4 || __GNUC_MINOR__ == 5) &&      \
+    __GNUC_PATCHLEVEL__ == 0 && defined(__OPTIMIZE__)
+        // Known to be broken in gcc 6.4 and 6.5 with optimizations
+        // Issue in gcc: https://gcc.gnu.org/bugzilla/show_bug.cgi?id=83116
+        CHECK(s1.size() == 4);
+        CHECK(s2.size() == 0);
+    #else
+        CHECK(s1.size() == s2.size());
+    #endif
+    }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("from_const_std_array_constructor")
 {
     const std::array<int, 4> arr = {1, 2, 3, 4};
@@ -475,6 +544,13 @@ TEST_CASE("from_const_std_array_constructor")
         CHECK((s.size() == narrow_cast<ptrdiff_t>(arr.size()) && s.data() == arr.data()));
     }
 
+    const std::array<AddressOverloaded, 4> ao_arr{};
+
+    {
+        span<const AddressOverloaded, 4> s{ao_arr};
+        CHECK((s.size() == narrow_cast<ptrdiff_t>(ao_arr.size()) && s.data() == ao_arr.data()));
+    }
+
 #ifdef CONFIRM_COMPILATION_ERRORS
     {
         span<const int, 2> s{arr};
@@ -504,6 +580,7 @@ TEST_CASE("from_const_std_array_constructor")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("from_std_array_const_constructor")
 {
     std::array<const int, 4> arr = {1, 2, 3, 4};
@@ -544,6 +621,7 @@ TEST_CASE("from_std_array_const_constructor")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("from_container_constructor")
 {
     std::vector<int> v = {1, 2, 3};
@@ -636,6 +714,7 @@ TEST_CASE("from_container_constructor")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("from_convertible_span_constructor")
 {
     {
@@ -673,6 +752,7 @@ TEST_CASE("from_convertible_span_constructor")
     #endif
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("copy_move_and_assignment")
 {
     span<int> s1;
@@ -694,6 +774,7 @@ TEST_CASE("copy_move_and_assignment")
     CHECK((s1.size() == 2 && s1.data() == &arr[1]));
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("first")
 {
     int arr[5] = {1, 2, 3, 4, 5};
@@ -732,6 +813,7 @@ TEST_CASE("first")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("last")
 {
     int arr[5] = {1, 2, 3, 4, 5};
@@ -769,6 +851,7 @@ TEST_CASE("last")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("subspan")
 {
     int arr[5] = {1, 2, 3, 4, 5};
@@ -850,6 +933,7 @@ TEST_CASE("subspan")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("at_call")
 {
     int arr[4] = {1, 2, 3, 4};
@@ -869,6 +953,7 @@ TEST_CASE("at_call")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("operator_function_call")
 {
     int arr[4] = {1, 2, 3, 4};
@@ -888,6 +973,7 @@ TEST_CASE("operator_function_call")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("iterator_default_init")
 {
     span<int>::iterator it1;
@@ -895,6 +981,7 @@ TEST_CASE("iterator_default_init")
     CHECK(it1 == it2);
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("const_iterator_default_init")
 {
     span<int>::const_iterator it1;
@@ -902,6 +989,7 @@ TEST_CASE("const_iterator_default_init")
     CHECK(it1 == it2);
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("iterator_conversions")
 {
     span<int>::iterator badIt;
@@ -924,6 +1012,7 @@ TEST_CASE("iterator_conversions")
     CHECK(cit3 == s.cend());
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("iterator_comparisons")
 {
     int a[] = {1, 2, 3, 4};
@@ -971,6 +1060,7 @@ TEST_CASE("iterator_comparisons")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("begin_end")
 {
     {
@@ -1026,6 +1116,7 @@ TEST_CASE("begin_end")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("cbegin_cend")
 {
     {
@@ -1078,6 +1169,7 @@ TEST_CASE("cbegin_cend")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("rbegin_rend")
 {
     {
@@ -1120,6 +1212,7 @@ TEST_CASE("rbegin_rend")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("crbegin_crend")
 {
     {
@@ -1159,6 +1252,7 @@ TEST_CASE("crbegin_crend")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("comparison_operators")
 {
     {
@@ -1279,6 +1373,7 @@ TEST_CASE("comparison_operators")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("as_bytes")
 {
     int a[] = {1, 2, 3, 4};
@@ -1309,6 +1404,7 @@ TEST_CASE("as_bytes")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("as_writeable_bytes")
 {
     int a[] = {1, 2, 3, 4};
@@ -1342,6 +1438,7 @@ TEST_CASE("as_writeable_bytes")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("fixed_size_conversions")
 {
     int arr[] = {1, 2, 3, 4};
@@ -1372,7 +1469,7 @@ TEST_CASE("fixed_size_conversions")
     {
         span<int> s = arr;
         auto f = [&]() {
-            span<int, 2> s2 = s;
+            const span<int, 2> s2 = s;
             static_cast<void>(s2);
         };
         CHECK_THROWS_AS(f(), fail_fast);
@@ -1382,7 +1479,7 @@ TEST_CASE("fixed_size_conversions")
 
     // you can convert statically
     {
-        const span<int, 2> s2 = {arr, 2};
+        const span<int, 2> s2 = {&arr[0], 2};
         static_cast<void>(s2);
     }
     {
@@ -1411,7 +1508,7 @@ TEST_CASE("fixed_size_conversions")
 #endif
     {
         auto f = [&]() {
-            span<int, 4> _s4 = {arr2, 2};
+            const span<int, 4> _s4 = {arr2, 2};
             static_cast<void>(_s4);
         };
         CHECK_THROWS_AS(f(), fail_fast);
@@ -1420,12 +1517,13 @@ TEST_CASE("fixed_size_conversions")
     // this should fail - we are trying to assign a small dynamic span to a fixed_size larger one
     span<int> av = arr2;
     auto f = [&]() {
-        span<int, 4> _s4 = av;
+        const span<int, 4> _s4 = av;
         static_cast<void>(_s4);
     };
     CHECK_THROWS_AS(f(), fail_fast);
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("interop_with_std_regex")
 {
     char lat[] = {'1', '2', '3', '4', '5', '6', 'E', 'F', 'G'};
@@ -1449,6 +1547,7 @@ TEST_CASE("interop_with_std_regex")
     CHECK(match[0].second == (f_it + 1));
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("interop_with_gsl_at")
 {
     int arr[5] = {1, 2, 3, 4, 5};
@@ -1462,3 +1561,4 @@ TEST_CASE("default_constructible")
     CHECK((std::is_default_constructible<span<int, 0>>::value));
     CHECK((!std::is_default_constructible<span<int, 42>>::value));
 }
+
