@@ -14,6 +14,19 @@
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+#ifdef _MSC_VER
+// blanket turn off warnings from CppCoreCheck from catch
+// so people aren't annoyed by them when running the tool.
+#pragma warning(disable : 26440 26426) // from catch deprecated
+#pragma warning(disable : 4996) // strided_span is in the process of being deprecated. 
+                                // Suppressing warnings until it is completely removed
+#endif
+
+#if __clang__ || __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
 #include <catch/catch.hpp> // for AssertionHandler, StringRef, CHECK, CHECK...
 
 #include <gsl/gsl_byte>   // for byte
@@ -60,17 +73,20 @@ TEST_CASE("span_section")
     const multi_span<int, 5, 10> av = as_multi_span(multi_span<int>{data}, dim<5>(), dim<10>());
 
     const strided_span<int, 2> av_section_1 = av.section({1, 2}, {3, 4});
+    CHECK(!av_section_1.empty());
     CHECK((av_section_1[{0, 0}] == 12));
     CHECK((av_section_1[{0, 1}] == 13));
     CHECK((av_section_1[{1, 0}] == 22));
     CHECK((av_section_1[{2, 3}] == 35));
 
     const strided_span<int, 2> av_section_2 = av_section_1.section({1, 2}, {2, 2});
+    CHECK(!av_section_2.empty());
     CHECK((av_section_2[{0, 0}] == 24));
     CHECK((av_section_2[{0, 1}] == 25));
     CHECK((av_section_2[{1, 0}] == 34));
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("strided_span_constructors")
 {
     // Check stride constructor
@@ -107,7 +123,7 @@ TEST_CASE("strided_span_constructors")
             CHECK(sav.bounds().strides() == multi_span_index<1>{1});
             CHECK(sav[1] == 2);
 
-#if _MSC_VER > 1800
+#if defined(_MSC_VER) && _MSC_VER > 1800
             // strided_span<const int, 1> sav_c{ {src}, {2, 1} };
             strided_span<const int, 1> sav_c{multi_span<const int>{src},
                                              strided_bounds<1>{2, 1}};
@@ -119,7 +135,7 @@ TEST_CASE("strided_span_constructors")
             CHECK(sav_c.bounds().strides() == multi_span_index<1>{1});
             CHECK(sav_c[1] == 2);
 
-#if _MSC_VER > 1800
+#if defined(_MSC_VER) && _MSC_VER > 1800
             strided_span<volatile int, 1> sav_v{src, {2, 1}};
 #else
             strided_span<volatile int, 1> sav_v{multi_span<volatile int>{src},
@@ -129,7 +145,7 @@ TEST_CASE("strided_span_constructors")
             CHECK(sav_v.bounds().strides() == multi_span_index<1>{1});
             CHECK(sav_v[1] == 2);
 
-#if _MSC_VER > 1800
+#if defined(_MSC_VER) && _MSC_VER > 1800
             strided_span<const volatile int, 1> sav_cv{src, {2, 1}};
 #else
             strided_span<const volatile int, 1> sav_cv{multi_span<const volatile int>{src},
@@ -149,7 +165,7 @@ TEST_CASE("strided_span_constructors")
             CHECK(sav_c.bounds().strides() == multi_span_index<1>{1});
             CHECK(sav_c[1] == 2);
 
-#if _MSC_VER > 1800
+#if defined(_MSC_VER) && _MSC_VER > 1800
             strided_span<const volatile int, 1> sav_cv{src, {2, 1}};
 #else
             strided_span<const volatile int, 1> sav_cv{multi_span<const volatile int>{src},
@@ -170,7 +186,7 @@ TEST_CASE("strided_span_constructors")
             CHECK(sav_v.bounds().strides() == multi_span_index<1>{1});
             CHECK(sav_v[1] == 2);
 
-#if _MSC_VER > 1800
+#if defined(_MSC_VER) && _MSC_VER > 1800
             strided_span<const volatile int, 1> sav_cv{src, {2, 1}};
 #else
             strided_span<const volatile int, 1> sav_cv{multi_span<const volatile int>{src},
@@ -271,6 +287,7 @@ TEST_CASE("strided_span_constructors")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("strided_span_slice")
 {
     std::vector<int> data(5 * 10);
@@ -297,6 +314,7 @@ TEST_CASE("strided_span_slice")
     CHECK(sav[4][9] == 49);
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("strided_span_column_major")
 {
     // strided_span may be used to accommodate more peculiar
@@ -329,6 +347,7 @@ TEST_CASE("strided_span_column_major")
     CHECK((cm_sec[{2, 1}] == 15));
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("strided_span_bounds")
 {
     int arr[] = {0, 1, 2, 3};
@@ -445,6 +464,7 @@ TEST_CASE("strided_span_bounds")
 #endif
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("strided_span_type_conversion")
 {
     int arr[] = {0, 1, 2, 3};
@@ -542,6 +562,8 @@ TEST_CASE("strided_span_type_conversion")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.4) // NO-FORMAT: attribute
 TEST_CASE("empty_strided_spans")
 {
     {
@@ -549,6 +571,7 @@ TEST_CASE("empty_strided_spans")
         strided_span<int, 1> empty_sav{empty_av, {0, 1}};
 
         CHECK(empty_sav.bounds().index_bounds() == multi_span_index<1>{0});
+        CHECK(empty_sav.empty());
         CHECK_THROWS_AS(empty_sav[0], fail_fast);
         CHECK_THROWS_AS(empty_sav.begin()[0], fail_fast);
         CHECK_THROWS_AS(empty_sav.cbegin()[0], fail_fast);
@@ -574,12 +597,14 @@ TEST_CASE("empty_strided_spans")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.1) // NO-FORMAT: attribute
 void iterate_every_other_element(multi_span<int, dynamic_range> av)
 {
     // pick every other element
 
     auto length = av.size() / 2;
-#if _MSC_VER > 1800
+#if defined(_MSC_VER) && _MSC_VER > 1800
     auto bounds = strided_bounds<1>({length}, {2});
 #else
     auto bounds = strided_bounds<1>(multi_span_index<1>{length}, multi_span_index<1>{2});
@@ -599,6 +624,7 @@ void iterate_every_other_element(multi_span<int, dynamic_range> av)
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
 TEST_CASE("strided_span_section_iteration")
 {
     int arr[8] = {4, 0, 5, 1, 6, 2, 7, 3};
@@ -616,6 +642,11 @@ TEST_CASE("strided_span_section_iteration")
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(r.11) // NO-FORMAT: attribute
+GSL_SUPPRESS(r.3) // NO-FORMAT: attribute
+GSL_SUPPRESS(r.5) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.1) // NO-FORMAT: attribute
 TEST_CASE("dynamic_strided_span_section_iteration")
 {
     auto arr = new int[8];
@@ -630,6 +661,9 @@ TEST_CASE("dynamic_strided_span_section_iteration")
     delete[] arr;
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.2) // NO-FORMAT: attribute // TODO: does not work
 void iterate_second_slice(multi_span<int, dynamic_range, dynamic_range, dynamic_range> av)
 {
     const int expected[6] = {2, 3, 10, 11, 18, 19};
@@ -656,6 +690,9 @@ void iterate_second_slice(multi_span<int, dynamic_range, dynamic_range, dynamic_
     }
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.2) // NO-FORMAT: attribute
 TEST_CASE("strided_span_section_iteration_3d")
 {
     int arr[3][4][2]{};
@@ -670,6 +707,11 @@ TEST_CASE("strided_span_section_iteration_3d")
     }
 }
 
+GSL_SUPPRESS(bounds.1) // NO-FORMAT: attribute
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(r.3) // NO-FORMAT: attribute
+GSL_SUPPRESS(r.5) // NO-FORMAT: attribute
+GSL_SUPPRESS(r.11) // NO-FORMAT: attribute
 TEST_CASE("dynamic_strided_span_section_iteration_3d")
 {
     const auto height = 12, width = 2;
@@ -702,6 +744,9 @@ TEST_CASE("dynamic_strided_span_section_iteration_3d")
     delete[] arr;
 }
 
+GSL_SUPPRESS(con.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.4) // NO-FORMAT: attribute
+GSL_SUPPRESS(bounds.2) // NO-FORMAT: attribute
 TEST_CASE("strided_span_conversion")
 {
     // get an multi_span of 'c' values from the list of X's
@@ -720,7 +765,7 @@ TEST_CASE("strided_span_conversion")
     auto d1 = narrow_cast<int>(sizeof(int)) * 12 / d2;
 
     // convert to 4x12 array of bytes
-    auto av = as_multi_span(as_bytes(as_multi_span(arr, 4)), dim(d1), dim(d2));
+    auto av = as_multi_span(as_bytes(as_multi_span(&arr[0], 4)), dim(d1), dim(d2));
 
     CHECK(av.bounds().index_bounds()[0] == 4);
     CHECK(av.bounds().index_bounds()[1] == 12);
@@ -755,3 +800,7 @@ TEST_CASE("strided_span_conversion")
         i++;
     }
 }
+
+#if __clang__ || __GNUC__
+#pragma GCC diagnostic pop
+#endif
